##
## Analyze attribution of deaths from FluView and excess deaths files
##
## CONFIDENTIAL AND PROPRIETARY
##
## CDC Excess Deaths Associated with COVID-19: https://www.cdc.gov/nchs/nvss/vsrr/covid19/excess_deaths.htm
##
##  Mortality Files: https://www.cdc.gov/nchs/covid19/covid-19-mortality-data-files.htm
##
##  Weekly Counts of Death by State and Select Causes: https://data.cdc.gov/NCHS/Weekly-Counts-of-Deaths-by-State-and-Select-Causes/3yf8-kanr
##
##  Click Export button in upper right of web page.  Select CSV
##
## NOTE: see fluview_copd.py for reading earlier format files from Jan 2021
##
## FluView Web Site:  https://gis.cdc.gov/grasp/fluview/mortality.html
##
## CDC Disease Burden of Influenza (influenza virus model): https://www.cdc.gov/flu/about/burden/index.html
##
## (C) 2021 by John F. McGowan, Ph.D. (ceo@mathematical-software.com)
##

# Python Standard Library imports
import os
import sys
import time
import traceback
import inspect
import datetime
import subprocess  # for run_cmd(...) etc.
import locale      # localization
import platform    # get Python version etc.
import shelve
import threading
import re          # regular expressions

# use greatest common denominator (GCD) function
from math import gcd
# use named tuples
from collections import namedtuple

# more Python standard libraries
import unittest
import io
from dateutil.parser import parse

# NumPy, Pandas
import numpy as np
import pandas as pd

# graphics libraries

# differences between operating systems
if sys.platform == "win32":
    # Microsoft Windows
    pass
elif sys.platform == "darwin":
    # Apple Mac OS X
    import matplotlib
    matplotlib.use("TkAgg")
elif sys.platform == "linux":
    # Linux Unix variants
    pass
else:
    pass

import matplotlib as mpl
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.ticker import FuncFormatter
from matplotlib.ticker import (MultipleLocator, AutoMinorLocator)
from matplotlib.font_manager import FontProperties

# date formatting
import matplotlib.dates as mdates
import matplotlib.cbook as cbook

# choose matplotlib graphics style
plt.style.use('ggplot')
#pd.options.display.mpl_style = 'default'

mpl.rcParams['font.size'] = 14
mpl.rcParams['font.weight'] = 'bold'
mpl.rcParams['axes.labelweight'] = 'bold'
mpl.rcParams['axes.titleweight'] = 'bold'

from matplotlib import pyplot as plt

assert sys.version_info[0] == 3, "This program requires Python 3"

SECONDS_PER_YEAR = 31_557_600

# the reference level for the CDC influenza deaths model
CDC_INFLUENZA_MODEL_ANNUAL_DEATHS_LEVEL = 55_000
CDC_INFLUENZA_MODEL_WEEKLY_DEATHS_LEVEL = (CDC_INFLUENZA_MODEL_ANNUAL_DEATHS_LEVEL/52.25)

PANDEMIC_YEAR = 2020

def week_ending_date_to_epoch(date_str):
    """
    convert Week Ending Date (e.g. 2019-12-29) to seconds since epoch started
    """
    
    date_pattern = '%Y-%m-%d' # Jan 2021 file format
    
    date_pattern = '%m/%d/%Y'  # April 15 2021 CDC file format
    epoch = int(time.mktime(time.strptime(date_str, date_pattern)))

    return epoch  # date_to_epoch(...)

def week_ending_date_to_years(date_str):
    """
    convert Week Ending Date (e.g. 2019-12-29) to years (e.g. 2019.99)
    """

    epoch = week_ending_date_to_epoch(date_str)
    years = epoch/SECONDS_PER_YEAR + 1970.0
    return years  # date_to_years(date_str)


#causes_df = pd.read_csv("Weekly_counts_of_death_by_jurisdiction_and_cause_of_death_jan_27_2021.csv", dtype=str)

causes_df = pd.read_csv("Weekly_Counts_of_Deaths_by_State_and_Select_Causes__2014-2019_downloaded_apr_15_2021.csv")

jurisdictions = np.unique(causes_df['Jurisdiction of Occurrence'])

us_p_and_i_deaths_series = causes_df[(causes_df['Jurisdiction of Occurrence'] == "United States")]['Influenza and pneumonia (J10-J18)']

us_p_and_i_deaths = np.array(list(map(lambda x: int(x),
                                      us_p_and_i_deaths_series)))

us_p_and_i_week_ending_date_series = causes_df[(causes_df['Jurisdiction of Occurrence'] == "United States")]['Week Ending Date']

us_p_and_i_dates = np.array(list(map(week_ending_date_to_years,
                                     us_p_and_i_week_ending_date_series)))

us_chronic_respiratory_deaths_series = causes_df[(causes_df['Jurisdiction of Occurrence'] == "United States") ]['Chronic lower respiratory diseases (J40-J47)']

us_chronic_respiratory_deaths = np.array(list(map(lambda x: int(x),
                                                  us_chronic_respiratory_deaths_series)))

us_chronic_respiratory_week_ending_date_series = causes_df[(causes_df['Jurisdiction of Occurrence'] == "United States") ]['Week Ending Date']

us_chronic_respiratory_dates = np.array(
    list(map(week_ending_date_to_years,
             us_chronic_respiratory_week_ending_date_series)))


if False:
    # stop here for debugging in ipython
    pass
else:

    # read FluView NCHS Pneumonia and Influenza Deaths Data
    NCHS_df = pd.read_csv("NCHSData06.csv")

    NCHS_years = np.array(NCHS_df["Year"])
    NCHS_weeks = np.array(NCHS_df["Week"])

    NCHS_year_dates = NCHS_years + NCHS_weeks/52.25

    NCHS_p_deaths = NCHS_df["Pneumonia Deaths"]
    NCHS_i_deaths = NCHS_df["Influenza Deaths"]
    NCHS_p_and_i_deaths = NCHS_p_deaths + NCHS_i_deaths


    # 'Week Ending Date'  for date/time
    # 'Number of Deaths'  for weekly deaths(?)
    #

    us_total_respiratory_deaths = us_p_and_i_deaths + us_chronic_respiratory_deaths

    fluview_deaths_by_year = {}
    
    fluview_i_deaths_by_year = {}

    cdc_x_p_deaths_by_year = {}

    cdc_x_copd_deaths_by_year = {}

    cdc_x_deaths_by_year = {}

    # Total Respiratory Deaths According to CDC Excess Deaths Web Site
    # combined P&I and chronic lower respiratory disease
    for index, date in enumerate(us_p_and_i_dates):
        calendar_year = int(date)
        # total respiratory deaths by year
        if calendar_year in cdc_x_deaths_by_year:
            cdc_x_deaths_by_year[calendar_year] += us_total_respiratory_deaths[index]
        else:
            cdc_x_deaths_by_year[calendar_year] = us_total_respiratory_deaths[index]
            
        # pneumonia and influenza deaths by year
        if calendar_year in cdc_x_p_deaths_by_year:
            cdc_x_p_deaths_by_year[calendar_year] += us_p_and_i_deaths[index]
        else:
            cdc_x_p_deaths_by_year[calendar_year] = us_p_and_i_deaths[index]

        # chronic lower respiratory/COPD deaths by year
        if calendar_year in cdc_x_copd_deaths_by_year:
            cdc_x_copd_deaths_by_year[calendar_year] += us_chronic_respiratory_deaths[index]
        else:
            cdc_x_copd_deaths_by_year[calendar_year] = us_chronic_respiratory_deaths[index]


    def ave_deaths_per_year(category_deaths_by_year):
        category_deaths_per_year = 0
        category_years = 0
        for year in category_deaths_by_year:
            if year < PANDEMIC_YEAR:
                category_deaths_per_year += category_deaths_by_year[year]
                category_years += 1
        category_deaths_per_year = category_deaths_per_year / category_years
        return int(category_deaths_per_year)  # ave_deaths_per_year

    cdc_x_deaths_per_year = ave_deaths_per_year(cdc_x_deaths_by_year)
    cdc_x_p_deaths_per_year = ave_deaths_per_year(cdc_x_p_deaths_by_year)
    cdc_x_copd_deaths_per_year = ave_deaths_per_year(cdc_x_copd_deaths_by_year)

    # Pneumonia and Influenza Deaths According to FluView web site
    #
    for index, date in enumerate(NCHS_year_dates):
        calendar_year = int(date)
        if calendar_year in fluview_deaths_by_year:
            fluview_deaths_by_year[calendar_year] += NCHS_p_and_i_deaths[index]
            fluview_i_deaths_by_year[calendar_year] += NCHS_i_deaths[index]
        else:
            fluview_deaths_by_year[calendar_year] = NCHS_p_and_i_deaths[index]
            fluview_i_deaths_by_year[calendar_year] = NCHS_i_deaths[index]

    fluview_deaths_per_year = ave_deaths_per_year(fluview_deaths_by_year)
    fluview_i_deaths_per_year = ave_deaths_per_year(fluview_i_deaths_by_year)

    print(f" YEAR  TOTAL RESPIRATORY DEATHS (CDC Excess Deaths Web Site/Project)")
    for calendar_year in cdc_x_deaths_by_year:
        print(f" {calendar_year}   {cdc_x_deaths_by_year[calendar_year]:,}")

    # Pneumonia and Influenza Deaths According to CDC Excess Deaths web site/project
    #

    print(f" YEAR  P&I DEATHS (CDC Excess Deaths Web Site/Project)")
    for calendar_year in cdc_x_p_deaths_by_year:
        print(f" {calendar_year}   {cdc_x_p_deaths_by_year[calendar_year]:,}")

    # Chronic Lower Respiratory Deaths Per Year
    #

    print(f" YEAR  CHRONIC LOWER RESPIRATORY DEATHS (CDC Excess Deaths Web Site/Project)")
    for calendar_year in cdc_x_copd_deaths_by_year:
        print(f" {calendar_year}   {cdc_x_copd_deaths_by_year[calendar_year]:,}")
        

    # Pneumonia and Influenza Deaths According to FluView web site/project
    #

    print(f" YEAR  PNEUMONIA AND INFLUENZA DEATHS (FluView Web Site)  DIFFERENCE FROM CDC EXCESS DEATHS PROJECT NUMBERS")
    for calendar_year in fluview_deaths_by_year:
        if calendar_year in cdc_x_deaths_by_year:
            diff = fluview_deaths_by_year[calendar_year] - cdc_x_deaths_by_year[calendar_year]
            print(f" {calendar_year}  {fluview_deaths_by_year[calendar_year]:,}" \
                  + f"                                                 {diff:,}")
        else:
            print(f" {calendar_year}  {fluview_deaths_by_year[calendar_year]:,}")

    print(f" YEAR  INFLUENZA VIRUS DEATHS (FluView Web Site)")
    for calendar_year in fluview_i_deaths_by_year:
        print(f" {calendar_year}  {fluview_i_deaths_by_year[calendar_year]:,}")


    cdc_influenza_model_level = CDC_INFLUENZA_MODEL_WEEKLY_DEATHS_LEVEL \
        * np.ones(NCHS_year_dates.shape)
    #
    # Plot comparing FluView numbers to sum of respiratory death categories
    # in CDC Excess Deaths data
    #

    fig1 = plt.figure()

    ax1 = plt.gca()
    # add thousands separator to xticks labels
    ax1.get_xaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{x:.2f}") )
    plt.xticks(fontsize=8)  # set font size of xticks labels

    # ax1.get_yaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{int(x):,}"))
    # plt.yticks(fontsize=8)

    plt.plot(us_p_and_i_dates, us_p_and_i_deaths, 'g+', \
             linewidth=3,
             label = f"(CDC Excess Deaths) Pneumonia and Influenza Deaths " \
             f"({cdc_x_p_deaths_per_year:,} per year on average 2014-2019)")
    
    ## CDC Influenza Model vs FluView Influenza Deaths
    ## Probably makes slide too busy for clear presentation.
    
    plt.plot(NCHS_year_dates, NCHS_i_deaths, 'm-',
             linewidth=3,
             label= f"FluView NCHS Influenza Virus Deaths " \
             + f"({fluview_i_deaths_per_year:,} per year on average 2014-2019)")
    
    # CDC Influenza Model
    plt.plot(NCHS_year_dates, cdc_influenza_model_level, 'k-', \
             linewidth=3,
             label="CDC Influenza Virus Weekly Deaths Model (55,000 per year on average 2014-2019)")

    #
    # where does FluView get the extra P and I deaths from???
    #
    plt.plot(us_chronic_respiratory_dates, us_chronic_respiratory_deaths, 'b+', \
             linewidth=3,
             label=f"(CDC Excess Deaths) Chronic Lower Respiratory Disease Deaths " \
             + f"({cdc_x_copd_deaths_per_year:,} per year on average 2014-2019)")
    
    plt.plot(us_p_and_i_dates, \
             us_total_respiratory_deaths, 'c+', \
             linewidth=1,
             markersize=5,
             label=f"(CDC Excess Deaths) Total Respiratory Deaths " \
             + f"({cdc_x_deaths_per_year:,} per year on average 2014-2019)")

    # FluView weekly Pneumonia and Influenza (P&I) deaths         
    plt.plot(NCHS_year_dates, NCHS_p_and_i_deaths, 'r-',
             linewidth=3,
             label= f"FluView Pneumonia and Influenza Deaths "
             + f"({fluview_deaths_per_year:,} per year on average 2014-2019)")
    
    plt.title("US CDC'S CONTRADICTORY PNEUMONIA AND INFLUENZA DEATH NUMBERS", fontsize=8)
    plt.xlabel("YEAR", fontsize=12)
    plt.ylabel("WEEKLY DEATHS (LOG SCALE)", fontsize=12)
    plt.yscale('log')  # select log scale
    
    # ax1.get_yaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{int(x):,}"))
    # plt.yticks(fontsize=8)
    
    plt.yticks([1, 10, 100, 1000, 2500, 5000, 10_000], fontsize=8)
    ax1.get_yaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{int(x):,}"))

    # add fancy grid
    plt.grid(True, 'major', color = '#666666', linestyle='-')
    plt.minorticks_on()
    plt.grid(True, 'minor', color= '#999999', linestyle='-', alpha=0.2)

    handles, labels = plt.gca().get_legend_handles_labels()
    #
    # Reorder labels to put fluview p&i deaths (scary red line) at top
    # of legend.
    #
    order = [-1] + list(range(0, len(labels)-1))
    plt.legend([handles[idx] for idx in order],
               [labels[idx] for idx in order],
               fontsize=6)
    plt.show()

    fig1.savefig("US_CDCs_Contradictory_Pneumonia_and_Influenza_Death_Numbers.jpg", dpi=300)


    # CDC Influenza Modle vs FluView Influenza Deaths
    fig2 = plt.figure()

    ax2 = plt.gca()
    # add thousands separator to xticks labels
    ax2.get_xaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{x:.2f}") )
    plt.xticks(fontsize=8)  # set font size of xticks labels

    ax2.get_yaxis().set_major_formatter( mpl.ticker.FuncFormatter(lambda x, p: f"{int(x):,}"))
    plt.yticks(fontsize=8)

    # FluView weekly deaths         
    plt.plot(NCHS_year_dates, NCHS_i_deaths, 'm-',
             linewidth=3,
             label="FluView NCHS Influenza Deaths (5-15,000 per year)")
    # CDC Influenza Model
    plt.plot(NCHS_year_dates, cdc_influenza_model_level, 'k-', \
             linewidth=3,
             label="CDC Influenza Weekly Deaths Model (55,000 per year)")

    plt.title("CDC US INFLUENZA WEEKLY DEATHS BY CATEGORY", fontsize=8)
    plt.xlabel("YEAR", fontsize=12)
    plt.ylabel("WEEKLY DEATHS", fontsize=12)
    plt.grid(True, 'major', color = '#666666', linestyle='-')
    plt.minorticks_on()
    plt.grid(True, 'minor', color= '#999999', linestyle='-', alpha=0.2)
    plt.legend(fontsize=10)
    plt.show()

    fig2.savefig("FluView_vs_CDC_Influenza_Model_Deaths.jpg", dpi=300)


    print(__file__, "ALL DONE!")

